;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - project-data.ss                                                #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

;; This variable contains all current project information
(define project-data-global '((project (label "Untitled Project")
                                       (path "")
                                       (selected #f)
                                       (collapsed #f))))

;; This boolean is true iff no change have been made since last save
(define no-change-since-last-save #t)

;; This functions allows to add a child to the project
(define (project-data-add-child child)
  ;; Helper
  (define (helper L)
    ;; Is the list empty?
    (cond ((null? L) L)
          ;; If the project has no children, add one
          ((not (assq 'children L)) (cons (list 'children child) L))
          ;; Otherwise, append the new child to the previous children list
          ((eq? (caar L) 'children)(cons (append (assq 'children L)(list child))(cdr L)))
          ;; Continue with the cdr
          (else (cons (car L) (helper (cdr L))))))
  ;; Function body
  (set! project-data-global (list (append (list 'project) (helper (cdar project-data-global))))))

;; This function adds and element whose parent is determined by parent-id
(define (project-data-add-element element parent-id)
  (define (cons-line line)
    (if (and (assq 'id (cdr line))
             (eq? (cadr (assq 'id (cdr line))) parent-id))
        (if (get-children line)
            (set-children line (append (get-children line) (list element)))
            (append (list (car line)) 
                    (list (list 'children element))
                    (cdr line)))
        line))
  (set! project-data-global (apply-function-on-data-list project-data-global cons-line)))

;; This function returns the element's line, given its id
(define (project-data-get-element-by-id id)
  ;; We don't know the output yet
  (let ((output '?))
    ;; This function is applied to each line
    (define (helper element)
      (if (and (not (eq? (car element) 'project))
             (eq? (get-property-value element 'id) id))
          ;; Is it the right line? Save it then. 
          (set! output element)))
    ;; Apply our procedure on each element
    (apply-procedure-on-data-list project-data-global helper)
    ;; Return the output
    output))
         
;; This function returns the element parent's line, or #f.
(define (project-data-get-parent element)
  ;; Get the parent line...
  (let ((temp-output (project-data-get-element-by-id (get-property-value element 'parent))))
    ;; Is it valid?
    (if (eq? temp-output '?) #f temp-output)))

;; This function sets the children to the given line
(define (set-children data-list children)
  (cond ((null? data-list) data-list)
        ((symbol? (car data-list)) (cons (car data-list) (set-children (cdr data-list) children)))
        ((eq? (caar data-list) 'children) (cons (cons 'children children) (set-children (cdr data-list) children)))
        (else (cons (car data-list) (set-children (cdr data-list) children)))))

;; This function gets the children of the given line
(define (get-children data-list)
  (if (assq 'children (cdr data-list))
      (cdr (assq 'children (cdr data-list)))
      #f))

;; This function applies a function to each line and returns the
;; built data-list.
(define (apply-function-on-data-list data-list function)
  ;; The helper function
  (define (helper data-list)
    (if (null? data-list)
        data-list
        (begin
          ;; Apply the right function
          (let* ((mod-current-line (function (car data-list)))
                 ;; First recursive call (for children)
                 (mod-current-children (if (assq 'children (cdar data-list)) (helper (cdr (assq 'children (cdr mod-current-line)))) #f)))
            ;; We build the result and call the second recursive call (for brothers)
            (cons (if mod-current-children
                      (set-children mod-current-line mod-current-children)
                      mod-current-line)
                  (helper (cdr data-list)))))))
  ;; The function body
  (helper data-list))
        
;; This is an improved version, which allows to delete lines.
(define (apply-function-on-data-list-improved data-list function)
  (define (set-children data-list children)
    (cond ((null? data-list) data-list)
          ((symbol? (car data-list)) (cons (car data-list) (set-children (cdr data-list) children)))
          ((and (eq? (caar data-list) 'children) (null? children) (set-children (cdr data-list) children)))
          ((eq? (caar data-list) 'children) (cons (cons 'children children) (set-children (cdr data-list) children)))
          (else (cons (car data-list) (set-children (cdr data-list) children)))))
  ;; The helper function
  (define (helper data-list)
    (if (null? data-list)
        data-list
        (begin
          ;; Apply the right function
          (let* ((mod-current-line (function (car data-list)))
                 ;; First recursive call (for children)
                 (mod-current-children (if (not (eq? mod-current-line 'nil))
                                           (if (assq 'children (cdar data-list))
                                               (helper (cdr (assq 'children (cdr mod-current-line))))
                                               #f)
                                           #f)))
            ;; We build the result and call the second recursive call (for brothers)
            (if (eq? mod-current-line 'nil)
                (helper (cdr data-list))
                (cons (if mod-current-children
                          (set-children mod-current-line mod-current-children)
                          mod-current-line)
                      (helper (cdr data-list))))))))
  ;; The function body
  (helper data-list))

;; This function applies a procedure to each line and returns void.
(define (apply-procedure-on-data-list data-list function)
  ;; The helper function
  (define (helper data-list)
    (if (null? data-list)
        (void)
        (begin
          ;; Apply the right function
          (function (car data-list))
          ;; First recursive call (for children)
          (if (assq 'children (cdar data-list)) 
              (helper (cdr (assq 'children (cdar data-list)))))
          ;; Second recursive call (for brothers)
          (helper (cdr data-list)))))
  ;; The function body
  (helper data-list))

;; This predicate returns #t if the line is a container
(define (container? line)
  (and (pair? line)
       (or (eq? (car line) 'frame)
           (eq? (car line) 'pane)
           (eq? (car line) 'panel)
           (eq? (car line) 'horizontal-pane)
           (eq? (car line) 'horizontal-panel)
           (eq? (car line) 'vertical-pane)
           (eq? (car line) 'vertical-panel)
           (eq? (car line) 'tab-panel)
           (eq? (car line) 'group-box-panel)
           (eq? (car line) 'grow-box-spacer-pane))))

;; This function updates the project path
(define (project-data-set-path path)
  (define (parse-line L)
    (cond ((null? L) '())
          ((symbol? (car L)) (cons (car L) (parse-line (cdr L))))
          ((eq? (caar L) 'path) (cons (list 'path path) (parse-line (cdr L))))
          (else (cons (car L) (parse-line (cdr L))))))
  (set! project-data-global (list (parse-line (car project-data-global))))
  (send hierarchy-widget update-in))

;; This function updates the project label
(define (project-data-set-label label)
  (define (parse-line L)
    (cond ((null? L) '())
          ((symbol? (car L)) (cons (car L) (parse-line (cdr L))))
          ((eq? (caar L) 'label) (cons (list 'label label) (parse-line (cdr L))))
          (else (cons (car L) (parse-line (cdr L))))))
  (set! project-data-global (list (parse-line (car project-data-global))))
  (send hierarchy-widget update-in))

;; This function returns the project path
(define (project-data-get-path)
  (define (parse-line L)
    (cond ((null? L) #f)
          ((symbol? (car L)) (parse-line (cdr L)))
          ((eq? (caar L) 'path) (cadar L))
          (else (parse-line (cdr L)))))
  (parse-line (car project-data-global)))

;; This function returns the property value, given a line
(define (get-property-value line property)
  (if (assq property (cdr line))
    (cadr (assq property (cdr line)))
    #f
  )
)

;; This very useful function does not have much to do with the project data, but
;; I had to find a place to declare it :)
(define (symbol-append . L)
    (string->symbol (apply string-append (map symbol->string L))))

