;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - toolbox-frame.ss                                               #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

;; We need to derive the frame class to override the on-close method.
(define toolbox-frame%
  (class frame%
    ;; We override on-close...
    ;(override on-close)
    ;; When on-close is called, we exit the project.
    (define/augment (on-close) (files-quit #f))
    (super-instantiate ())))

;; Toolbox frame definition...
(define toolbox-frame (new toolbox-frame%
                           (label "Toolbox")
                           (parent #f)
                           (width 355)
                           (x 5)
                           (y 5)
                           (style '(no-resize-border))))

;; Main menu bar definition...
(define toolbox-frame-menu (new menu-bar%
                             (parent toolbox-frame)))

;; File menu definition...
(define toolbox-frame-file-menu (new menu%
                                  (label "File") 
                                  (parent toolbox-frame-menu) 
                                  (help-string "File menu")))

;; "New Project..." item definition...
(new menu-item%
     (label "&New Project...")
     (shortcut #\N)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (files-new))))

;; "New Project..." item definition...
(new menu-item%
     (label "&Open Project...")
     (shortcut #\O)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (files-open))))

;; "New Project..." item definition...
(new separator-menu-item% (parent toolbox-frame-file-menu))

;; "Save" item definition...
(new menu-item%
     (label "&Save")
     (shortcut #\S)
     (parent toolbox-frame-file-menu)
     (callback (lambda (menu-item control-event) (files-save))))

;; "Save as..." item definition...
(new menu-item%
     (label "&Save as...")
     (parent toolbox-frame-file-menu)
     (callback (lambda (menu-item control-event) (files-save-as))))

;; "Save as..." item definition...
(new menu-item%
     (label "&Export as a Scheme File...")
     (shortcut #\E)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (generate-code-to-file))))

(new separator-menu-item% (parent toolbox-frame-file-menu))

;; "Quit" item definition...
(new menu-item%
     (label "E&xit")
     (shortcut #\Q)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (toolbox-frame-file-menu-on-quit))))

;; Edit menu definition...
(define toolbox-frame-edit-menu (new menu%
                                  (label "Edit") 
                                  (parent toolbox-frame-menu) 
                                  (help-string "Edit menu")))

;; "Cut" item definition...
(send (new menu-item%
           (label "C&ut")
           (shortcut #\X)
           (parent toolbox-frame-edit-menu)
           (callback (lambda (item event) (void)))) enable #f)

;; "Copy" item definition...
(send (new menu-item%
     (label "&Copy")
     (shortcut #\C)
     (parent toolbox-frame-edit-menu)
     (callback (lambda (item event) (void)))) enable #f)

;; "Paste" item definition...
(send (new menu-item%
     (label "&Paste")
     (shortcut #\V)
     (parent toolbox-frame-edit-menu)
     (callback (lambda (item event) (void)))) enable #f)

;; "Clear" item definition...
(send (new menu-item%
     (label "Clea&r")
     (parent toolbox-frame-edit-menu)
     (callback (lambda (item event) (void)))) enable #f)

(new separator-menu-item% (parent toolbox-frame-edit-menu))

;; "Clear" item definition...
(send (new menu-item%
     (label "Select All")
     (parent toolbox-frame-edit-menu)
     (callback (lambda (item event) (void)))) enable #f)

;; Windows menu definition...
(define toolbox-frame-windows-menu (new menu%
                                  (label "Windows") 
                                  (parent toolbox-frame-menu) 
                                  (help-string "Windows menu")))

;; "Hierarchy frame" item definition...
(define properties-frame-windows-hierarchy-frame-menu-item 
  (new checkable-menu-item%
              (label "Hierarchy frame")
              (parent toolbox-frame-windows-menu)
              (callback (lambda (item event) 
                          (if (send item is-checked?)
                              (begin
                                (send hierarchy-frame show #t)
                                (send item check #t))
                              (begin
                                (send hierarchy-frame show #f)
                                (send item check #f)))))))

;; Perform necessary updates...
(send properties-frame-windows-hierarchy-frame-menu-item enable #t)
(send properties-frame-windows-hierarchy-frame-menu-item check #t)

;; "Properties frame" item definition...
(define properties-frame-windows-properties-frame-menu-item 
(new checkable-menu-item%
     (label "Properties frame")
     (parent toolbox-frame-windows-menu)
     (callback (lambda (item event) 
                 (if (send item is-checked?)
                     (begin
                       (send properties-frame show #t)
                       (send item check #t))
                     (begin
                       (send properties-frame show #f)
                       (send item check #f)))))))

;; Perform necessary updates...
(send properties-frame-windows-properties-frame-menu-item enable #t)
(send properties-frame-windows-properties-frame-menu-item check #t)

;; Help menu definition...
(define toolbox-frame-help-menu (new menu%
                                  (label "Help") 
                                  (parent toolbox-frame-menu) 
                                  (help-string "Help menu")))

;; "Online Help..." item definition...
(new menu-item%
     (label "Online &Help...")
     (shortcut #\H)
     (parent toolbox-frame-help-menu)
     (callback (lambda (item event) (help-online-help))))

;; "MrEd Help (from PLT)..." item definition...
(new menu-item%
     (label "&MrEd v206p1 Help...")
     (parent toolbox-frame-help-menu)
     (callback (lambda (item event) (help-mred-help))))

(new separator-menu-item% (parent toolbox-frame-help-menu))

;; "About MrEd Designer..." item definition...
(new menu-item%
     (label "&About MrEd Designer...")
     (shortcut #\?)
     (parent toolbox-frame-help-menu)
     (callback (lambda (item event) (help-about-dialog))))

;; Lists containing callbacks and corresponding pictures
;; for the toolbox frame buttons...
(define toolbox-frame-buttons-first-list
  '(( toolbox-frame-button-frame-callback .  "frame.png" )
    ( toolbox-frame-button-pane-callback . "pane.png" )
    ( toolbox-frame-button-horizontal-pane-callback . "horizontal-pane.png" )
    ( toolbox-frame-button-vertical-pane-callback . "vertical-pane.png" )
    ( toolbox-frame-button-panel-callback . "panel.png" )
    ( toolbox-frame-button-horizontal-panel-callback . "horizontal-panel.png" )
    ( toolbox-frame-button-vertical-panel-callback . "vertical-panel.png" )
    ( toolbox-frame-button-grow-box-spacer-pane-callback . "grow-box-spacer-pane.png" )
    ( toolbox-frame-button-tab-panel-callback . "tab-panel.png" )
    ( toolbox-frame-button-group-box-panel-callback . "group-box-panel.png" )))

(define toolbox-frame-buttons-second-list
  '(( toolbox-frame-button-message-callback . "message.png" )
    ( toolbox-frame-button-radio-box-callback . "radio-box.png" )
    ( toolbox-frame-button-check-box-callback . "check-box.png" )
    ( toolbox-frame-button-button-callback . "button.png" )
    ( toolbox-frame-button-canvas-callback . "canvas.png" )
    ( toolbox-frame-button-choice-callback . "choice.png" )
    ( toolbox-frame-button-gauge-callback . "gauge.png" )
    ( toolbox-frame-button-list-box-callback . "list-box.png" )
    ( toolbox-frame-button-slider-callback . "slider.png" )
    ( toolbox-frame-button-text-field-callback . "text-field.png" )))
    
;; The vertical pane
(define toolbox-frame-vertical-pane (new vertical-pane% (parent toolbox-frame) (alignment '(right top)) (border 3)))

;; The first group box panel
(define toolbox-frame-first-gbp (new group-box-panel% (label "Containers") (parent toolbox-frame-vertical-pane)))
;; The first horizontal pane
(define toolbox-frame-first-horizontal-pane (new horizontal-pane% (parent toolbox-frame-first-gbp)))

;; We fill it
(define (toolbox-frame-make-first-buttons list)
  (if (not (null? list))
      (begin
        (new button%
             (label (make-object bitmap% 
                                 (build-path "images" "icons" "24x24" (cdar list))
                                 'png (make-object color% 255 255 255))) 
             (parent toolbox-frame-first-horizontal-pane)
             (style '(border))
             (callback (lambda (button event) ((eval (caar list)) button event))))
        (toolbox-frame-make-first-buttons (cdr list)))))

;; The second group box panel
(define toolbox-frame-second-gbp (new group-box-panel% (label "Controls") (parent toolbox-frame-vertical-pane)))
;; The second horizontal pane
(define toolbox-frame-second-horizontal-pane (new horizontal-pane% (parent toolbox-frame-second-gbp)))

;; We fill it
(define (toolbox-frame-make-second-buttons list)
  (if (not (null? list))
          (begin
            (new button%
                 (label (make-object bitmap% 
                                     (build-path "images" "icons" "24x24" (cdar list))
                                     'png (make-object color% 255 255 255))) 
                 (parent toolbox-frame-second-horizontal-pane)
                 (style '(border))
                 (callback (lambda (button event) ((eval (caar list)) button event))))
                 (toolbox-frame-make-second-buttons (cdr list)))))

(toolbox-frame-make-first-buttons toolbox-frame-buttons-first-list)
(toolbox-frame-make-second-buttons toolbox-frame-buttons-second-list)

;; Callback called when the user quits the application :
(define (toolbox-frame-file-menu-on-quit)
  (files-quit #t))
 
;; Callback called when user clicks the "frame" button :
(define (toolbox-frame-button-frame-callback button event)
  
  ;; A parser which removes the certain values
  (define (parse-list L)
    (cond ((null? L) '())
          ((eq? (caar L) 'necessary)(parse-list (cdr L)))
          ((eq? (caar L) 'optional)(parse-list (cdr L)))
          ((eq? (cadar L) 'na) (parse-list (cdr L)))
          (else (cons (car L) (parse-list (cdr L))))))
  
  (let ((selected-element (send hierarchy-widget get-selected-element)))
    ;; If the parent is another frame...
    (cond ((and selected-element (eq? (car selected-element) 'frame))
           ;; Then call the default callback...
           (default-callback 'frame 'frame% frame-default-values button))
          ;; If the project root is selected in the hierarchy widget...
          ((or (not selected-element) (eq? (car selected-element) 'project))
           
           ;; ...create the frame...
           (let ((frame (eval (append (list 'new) 
                                      (list 'frame%) 
                                      (parse-list (cdr frame-default-values))))))
             
             ;; ...add the frame to the project data...
             (project-data-add-child (append (list 'frame)
                                             (list (list 'pointer frame))
                                             (list (list 'selected #f))
                                             (list (list 'collapsed #f))
                                             (list (list 'id (gensym 'frame-)))
                                             (cdr frame-default-values)))
             
             ;; ...warn the hierarchy widget...
             (send hierarchy-widget update-in)
             
             ;; ...update the no-change-since-last-save variable...
             (set! no-change-since-last-save #f)
             
             ;; ...show the frame...
             (send frame show #t)
             
             ;; ...and refresh the button.
             (send button refresh)))
        
          ;; Otherwise...
          (else
           ;; ...display an error.
           (message-box 
            "Error" 
            "You cannot create a frame because the currently selected element in the hierarchy frame is neither a frame nor the project root."
            toolbox-frame '(ok stop))))))

;; Callback called when user clicks the "pane" button :
(define (toolbox-frame-button-pane-callback button event)
  (default-callback 'pane 'pane% pane-default-values button))

;; Callback called when user clicks the "horizontal pane" button :
(define (toolbox-frame-button-horizontal-pane-callback button event)
  (default-callback 'horizontal-pane 'horizontal-pane% horizontal-pane-default-values button))

;; Callback called when user clicks the "vertical pane" button :
(define (toolbox-frame-button-vertical-pane-callback button event)
  (default-callback 'vertical-pane 'vertical-pane% vertical-pane-default-values button))

;; Callback called when user clicks the "panel" button :
(define (toolbox-frame-button-panel-callback button event)
  (default-callback 'panel 'panel% panel-default-values button))

;; Callback called when user clicks the "horizontal panel" button :
(define (toolbox-frame-button-horizontal-panel-callback button event)
  (default-callback 'horizontal-panel 'horizontal-panel% horizontal-panel-default-values button))

;; Callback called when user clicks the "vertical panel" button :
(define (toolbox-frame-button-vertical-panel-callback button event)
  (default-callback 'vertical-panel 'vertical-panel% vertical-panel-default-values button))

;; Callback called when user clicks the "tab panel" button :
(define (toolbox-frame-button-tab-panel-callback button event)
  (default-callback 'tab-panel 'tab-panel% tab-panel-default-values button))

;; Callback called when user clicks the "group box panel" button :
(define (toolbox-frame-button-group-box-panel-callback button event)
  (default-callback 'group-box-panel 'group-box-panel% group-box-panel-default-values button))

;; Callback called when user clicks the "grow box spacer pane" button :
(define (toolbox-frame-button-grow-box-spacer-pane-callback button event)
  (default-callback 'grow-box-spacer-pane 'grow-box-spacer-pane% grow-box-spacer-pane-default-values button))

;; Callback called when user clicks the "message" button :
(define (toolbox-frame-button-message-callback button event)
  (default-callback 'message 'message% message-default-values button))

;; Callback called when user clicks the "button" button :
(define (toolbox-frame-button-button-callback button event)
  (default-callback 'button 'button% button-default-values button))

;; Callback called when user clicks the "radio-box" button :
(define (toolbox-frame-button-radio-box-callback button event)
  (default-callback 'radio-box 'radio-box% radio-box-default-values button))

;; Callback called when user clicks the "check-box" button :
(define (toolbox-frame-button-check-box-callback button event)
  (default-callback 'check-box 'check-box% check-box-default-values button))

;; Callback called when user clicks the "slider" button :
(define (toolbox-frame-button-slider-callback button event)
  (default-callback 'slider 'slider% slider-default-values button))

;; Callback called when user clicks the "gauge" button :
(define (toolbox-frame-button-gauge-callback button event)
  (default-callback 'gauge 'gauge% gauge-default-values button))

;; Callback called when user clicks the "list-box" button :
(define (toolbox-frame-button-list-box-callback button event)
  (default-callback 'list-box 'list-box% list-box-default-values button))

;; Callback called when user clicks the "choice" button :
(define (toolbox-frame-button-choice-callback button event)
  (default-callback 'choice 'choice% choice-default-values button))

;; Callback called when user clicks the "text-field" button :
(define (toolbox-frame-button-text-field-callback button event)
  (default-callback 'text-field 'text-field% text-field-default-values button))

;; Callback called when user clicks the "canvas" button :
(define (toolbox-frame-button-canvas-callback button event)
  (default-callback 'canvas 'canvas% canvas-default-values button))

;; This is a default callback for each button
(define (default-callback type name default-values btn)
  ;; A parser which removes the certain values
  (define (parse-list L)
    (cond ((null? L) '())
          ((eq? (caar L) 'parent)(cons (list 'parent (cadr (assq 'pointer (cdr (send hierarchy-widget get-selected-element)))))
                                       (parse-list (cdr L))))
          ((eq? (caar L) 'necessary)(parse-list (cdr L)))
          ((eq? (caar L) 'optional)(parse-list (cdr L)))
          ((eq? (cadar L) 'na) (parse-list (cdr L)))
          (else (cons (car L) (parse-list (cdr L))))))
  
  (if (container? (send hierarchy-widget get-selected-element))
      (begin        
        ;; Create the element
        (let ((element (eval (append (list 'new) 
                                     (list name) 
                                     (parse-list (cdr default-values))))))
          
          ;; We need a little parser to replace the "parent" element in the default values...
          (letrec ((parse-list (lambda (L)
                                 (cond ((null? L) L)
                                       ((eq? (caar L) 'parent) 
                                        (cons (list 'parent (send hierarchy-widget get-selected-element-id)) (parse-list (cdr L))))
                                       (else (cons (car L) (parse-list (cdr L))))))))
                                             
            ;; Add the button to the project data
            (project-data-add-element (append (list type)
                                              (list (list 'pointer element))
                                              (list (list 'selected #f))
                                              (list (list 'collapsed #f))
                                              (list (list 'id (gensym (string->symbol (string-append (symbol->string type) "-")))))
                                              (parse-list (cdr default-values)))
                                      (send hierarchy-widget get-selected-element-id)))
          
          ;; If the element is a frame...
          (if (eq? type 'frame)
              ;; ...show it!
              (send element show #t))
          
          ;; We warn the hierarchy widget...
          (send hierarchy-widget update-in)
          
          ;; At least one change have been made
          (set! no-change-since-last-save #f)
          
          ;; Refresh the button because of a bug on MacOS X
          (send btn refresh)))
      (message-box "Error" "You cannot instantiate this widget because the currently selected element in the hierarchy frame is not a container." toolbox-frame '(ok stop))))

;; We add the "Generate Code..." button...
(new button%
     (label "Generate Code...") 
     (parent toolbox-frame-vertical-pane)
     (min-width 150)
     (callback (lambda (button event) (generate-code-callback))))

;; And finally, we show the toolbox frame...
(send toolbox-frame show #t)